@@ -57,8 +57,7 @@ type operation_t = CommandOp of command_t | EntryOp of entry_operation_t
 type colorable_object_t = | Help | Timed_default | Timed_current | Untimed_reminder
                           | Timed_date | Selection_info | Description | Status 
                           | Calendar_labels | Calendar_level1 | Calendar_level2
-                          | Calendar_level3 | Calendar_selection | Calendar_today 
-                          | Left_divider | Right_divider 
+                          | Calendar_level3 | Calendar_today | Left_divider | Right_divider 
                           | Timed_reminder1 | Timed_reminder2 | Timed_reminder3
                           | Timed_reminder4
 
@@ -123,6 +122,11 @@ let number_weeks = ref false
 let home_sticky = ref true
 (* width of calendar/untimed reminder windows *)
 let untimed_window_width = ref 40
+(* trigger advance warning of reminders? *)
+let advance_warning = ref false
+(* render untimed reminders in bold? *)
+let untimed_bold = ref true
+
 (* List of included rc files *)
 let included_rcfiles : (string list) ref = ref []
 
@@ -553,6 +557,10 @@ let parse_line line_stream =
          parse_set "home_sticky" home_sticky bool_of_string "Expected a boolean string after "
       | [< 'Ident "untimed_window_width" >] ->
          parse_set "untimed_window_width" untimed_window_width int_of_string "Expected an integral string after "
+      | [< 'Ident "advance_warning" >] ->
+         parse_set "advance_warning" advance_warning bool_of_string "Expected a boolean string after "
+      | [< 'Ident "untimed_bold" >] ->
+         parse_set "untimed_bold" untimed_bold bool_of_string "Expected a boolean string after "
       | [< >] ->
          config_failwith ("Unmatched variable name after \"set\"")
       end
@@ -572,7 +580,11 @@ let parse_line line_stream =
       | [< 'Ident "calendar_level1" >]  -> parse_color "calendar_level1" Calendar_level1
       | [< 'Ident "calendar_level2" >]  -> parse_color "calendar_level2" Calendar_level2
       | [< 'Ident "calendar_level3" >]  -> parse_color "calendar_level3" Calendar_level3
-      | [< 'Ident "calendar_selection" >] -> parse_color "calendar_selection" Calendar_selection
+      | [< 'Ident "calendar_selection" >] -> 
+         begin
+         Printf.fprintf stderr "Warning: colorable object \"calendar_selection\" has been ";
+         Printf.fprintf stderr "deprecated.\nPlease remove this reference from the wyrdrc file.\n"
+         end
       | [< 'Ident "calendar_today" >]   -> parse_color "calendar_today" Calendar_today
       | [< 'Ident "left_divider" >]     -> parse_color "left_divider" Left_divider
       | [< 'Ident "right_divider" >]    -> parse_color "right_divider" Right_divider
